[{"locale":"es"},{"key":"bbl","names":["bbl.","bbl"],"mappings":{"default":{"default":"barrile"}},"category":"volume"},{"key":"cu","names":["cu","cu."],"mappings":{"default":{"default":"cúbico"}},"category":"volume"},{"key":"fl. oz.","names":["fl. oz."],"mappings":{"default":{"default":"onzas líquidas"}},"category":"volume"},{"key":"gal","names":["gal","gal."],"mappings":{"default":{"default":"galone"}},"category":"volume"},{"key":"pt","names":["pt","pt."],"mappings":{"default":{"default":"pinta"}},"category":"volume"},{"key":"qt","names":["qt","qt."],"mappings":{"default":{"default":"quart"}},"category":"volume"},{"key":"tbsp","names":["tbsp","tbsp."],"mappings":{"default":{"default":"cuchara"}},"category":"volume"},{"key":"tsp","names":["tsp","tsp."],"mappings":{"default":{"default":"cucharilla"}},"category":"volume"},{"key":"cc","names":["cc"],"mappings":{"default":{"default":"centímetros cúbicos"}},"category":"volume"},{"key":"kl","names":["kl"],"mappings":{"default":{"default":"kilolitro"}},"category":"volume"},{"key":"l","names":["l"],"mappings":{"default":{"default":"litro"}},"category":"volume"},{"key":"ml","names":["ml"],"mappings":{"default":{"default":"mililitro"}},"category":"volume"}]
